@@ -4,24 +4,30 @@ use strict;
 use Net::SSH::Perl::Buffer;
 use Net::SSH::Perl::Constants qw(
     SSH_SMSG_STDOUT_DATA
+    SSH_SMSG_EXITSTATUS
     SSH_CMSG_STDIN_DATA
     SSH_CMSG_EOF );
 
+use constant CHUNK_SIZE => 32000;
+
 use Tie::Handle;
 use base qw( Tie::Handle );
 
 sub TIEHANDLE {
     my $class = shift;
-    my($mode, $ssh, $r_exit) = @_;
+    my($mode, $ssh) = @_;
     my $read = $mode =~ /^[rR]/;
-    my $handle = bless { ssh => $ssh, exit => $r_exit }, $class;
+    my $handle = bless { ssh => $ssh }, $class;
     if ($read) {
-        my $incoming = $handle->{incoming} = Net::SSH::Perl::Buffer->new( MP => 'SSH1' );
+        my $incoming = $handle->{incoming} =
+            Net::SSH::Perl::Buffer->new( MP => 'SSH1' );
         $ssh->register_handler(SSH_SMSG_STDOUT_DATA, sub {
             my($ssh, $packet) = @_;
             $incoming->append($packet->get_str);
             $ssh->break_client_loop;
         });
+        $ssh->register_handler(SSH_SMSG_EXITSTATUS,
+            sub { $handle->{exit} = $_[1]->get_int32 });
     }
     $handle;
 }
@@ -29,6 +35,7 @@ sub TIEHANDLE {
 sub READ {
     my $h = shift;
     my $buf = $h->{incoming};
+    $_[0] = undef, return 0 unless $buf->length || !$h->EOF;
     while (!$buf->length) {
         $h->{ssh}->_start_interactive;
         $_[0] = undef, return 0 unless $buf->length;
@@ -41,13 +48,17 @@ sub READ {
 sub WRITE {
     my $h = shift;
     my($data) = @_;
-    my $packet = $h->{ssh}->packet_start(SSH_CMSG_STDIN_DATA);
-    $packet->put_str($data);
-    $packet->send;
-    length($data);
+    my $len = length($data);
+    while ($data) {
+        my $chunk = substr($data, 0, CHUNK_SIZE, '');
+        my $packet = $h->{ssh}->packet_start(SSH_CMSG_STDIN_DATA);
+        $packet->put_str($chunk);
+        $packet->send;
+    }
+    $len;
 }
 
-sub EOF { defined ${$_[0]->{exit}} ? 1 : 0 }
+sub EOF { defined $_[0]->{exit} ? 1 : 0 }
 
 sub CLOSE {
     my $h = shift;
@@ -57,6 +68,7 @@ sub CLOSE {
         $packet->send;
         $ssh->_start_interactive;
     }
+    1;
 }
 
 1;
